'use strict';

const _ = require('lodash');
const Promise = require('bluebird');
const {gunzip} = require('zlib');
const ReleaseProto = require('./../../../_proto/hapi/release/release_pb').Release;

const propMap = {
    name: 'NAME',
    status: 'STATUS',
    version: 'VERSION',
    modifiedAt: 'MODIFIED_AT',
    createdAt: 'CREATED_AT'
};

class Release {

    constructor(data){
        this._name = data[propMap.name];
        this._version = data[propMap.version];
        this._status = data[propMap.status];
        this._modifiedAt = data[propMap.modifiedAt];
        this._createdAt = data[propMap.createdAt];
        this._rawName = data.rawName;
        this._lastRevisionData = data.dataAsBase64;
        this._history = [this._getStriptedData()];
    }

    get name(){
        return this._name;
    }

    updateRelease(release){
        this._history.push(release._getStriptedData());
        if(release._createdAt){
            this._createdAt = release._createdAt;
        }
        if(parseInt(this._version) < parseInt(release._version)){
            this._version = release._version;
            this._modifiedAt = release._modifiedAt;
            this._status = release._status;
            this._lastRevisionData = release._lastRevisionData;
        } 
    }

    _getStriptedData(){
        return {
            name: this._name,
            version: this._version,
            status: this._status,
            modifiedAt: this._modifiedAt,
            createdAt: this._createdAt,
        };
    }

    getBasicData(){
        return Object.assign(this._getStriptedData(), {
            history: this._history,
            chart: `${_.get(this._lastRevisionData, 'chart.metadata.name')}-${_.get(this._lastRevisionData, 'chart.metadata.version')}`,
            icon: _.get(this._lastRevisionData, 'chart.metadata.icon'),
            description: _.get(this._lastRevisionData, 'chart.metadata.description'), 
            sourcesList: _.get(this._lastRevisionData, 'chart.metadata.sourcesList')
        });
    }

    getFullData(){
        return Object.assign(this.getBasicData(), {
            lastRevisionData: this._lastRevisionData
        });
    }

    finalize(){
        const data = Buffer.from(this._lastRevisionData, 'base64');
        return Promise.fromCallback((cb) => gunzip(data, cb))
        .then((buffer) => ReleaseProto.deserializeBinary(new Uint8Array(Buffer.from(buffer))))
        .then(protoRelease => {
            this._lastRevisionData = protoRelease.toObject();
        });
    }
}

module.exports = Release;