'use strict';

const chai = require('chai');
const sinonChai = require('sinon-chai');
const _ = require('lodash');
const proxyquire = require('proxyquire');
const Promise = require('bluebird');
const expect = chai.expect;
chai.use(sinonChai);
const Release = require('./../release');

function getFakeBase64Data(){
    return ',base64;';
}

describe('Release', () => {
    describe('Finalize', () => {
        it('Should success read all data and create proto release', () => {
            const proxyOpt = {
                './../../../_proto/hapi/release/release_pb': {
                    Release: {
                        deserializeBinary(){
                            return {
                               toObject(){
                                   return 'deserializedProto';                                   
                               } 
                            };
                        },
                    }
                },
                'zlib': {
                    gunzip: (data, cb) => {
                        return cb(null, data);
                    }
                }
            };
            const ReleaseProxy = proxyquire('./../release', proxyOpt);
            const r = new ReleaseProxy({
                'NAME': 'r1',
                'VERSION': '1',
                dataAsBase64: getFakeBase64Data()
            });

            return r.finalize()
            .then(() => {
                const data = r.getFullData();
                expect(data.lastRevisionData).to.be.deep.equal('deserializedProto');
            });
        });
    });
});